<?php

namespace Moblyze\Waterway\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Moblyze\Waterway\Http\Requests\WaterwayRequest;
use Moblyze\Waterway\Interfaces\WaterwayRepositoryInterface;
use Moblyze\Waterway\Models\Waterway;
use Moblyze\Waterway\Forms\Waterway as Form;

/**
 * APIController  class for waterway.
 */
class WaterwayAPIController extends BaseController
{

    /**
     * Initialize waterway resource controller.
     *
     * @param type WaterwayRepositoryInterface $waterway
     *
     * @return null
     */
    public function __construct(WaterwayRepositoryInterface $waterway)
    {
        parent::__construct();
        $this->repository = $waterway;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Waterway\Repositories\Criteria\WaterwayResourceCriteria::class);
    }

    /**
     * Display a list of waterway.
     *
     * @return Response
     */
    public function index(WaterwayRequest $request)
    {
        return $this->repository
            ->setPresenter(\Moblyze\Waterway\Repositories\Presenter\WaterwayPresenter::class)
            ->paginate();
    }

    /**
     * Display waterway.
     *
     * @param Request $request
     * @param Model   $waterway
     *
     * @return Response
     */
    public function show(WaterwayRequest $request, Waterway $waterway)
    {
        return $waterway->setPresenter(\Moblyze\Waterway\Repositories\Presenter\WaterwayListPresenter::class);
        ;
    }

    /**
     * Create new waterway.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(WaterwayRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('waterway::waterway.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('waterway/waterway/' . $waterway->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('waterway/waterway');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the waterway.
     *
     * @param Request $request
     * @param Model   $waterway
     *
     * @return Response
     */
    public function update(WaterwayRequest $request, Waterway $waterway)
    {
        try {
            $data = $request->all();

            $waterway->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('waterway::waterway.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('waterway/waterway/' . $waterway->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('waterway/waterway/' . $waterway->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the waterway.
     *
     * @param Model   $waterway
     *
     * @return Response
     */
    public function destroy(WaterwayRequest $request, Waterway $waterway)
    {
        try {
            $waterway->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('waterway::waterway.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('waterway/waterway/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('waterway/waterway/' . $waterway->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
