<?php

namespace Moblyze\Waterway\Policies;

use Litepie\User\Contracts\UserPolicy;
use Moblyze\Waterway\Models\Waterway;

class WaterwayPolicy
{

    /**
     * Determine if the given user can view the waterway.
     *
     * @param UserPolicy $user
     * @param Waterway $waterway
     *
     * @return bool
     */
    public function view(UserPolicy $user, Waterway $waterway)
    {
        if ($user->canDo('waterway.waterway.view') && $user->isAdmin()) {
            return true;
        }

        return $waterway->user_id == user_id() && $waterway->user_type == user_type();
    }

    /**
     * Determine if the given user can create a waterway.
     *
     * @param UserPolicy $user
     * @param Waterway $waterway
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('waterway.waterway.create');
    }

    /**
     * Determine if the given user can update the given waterway.
     *
     * @param UserPolicy $user
     * @param Waterway $waterway
     *
     * @return bool
     */
    public function update(UserPolicy $user, Waterway $waterway)
    {
        if ($user->canDo('waterway.waterway.edit') && $user->isAdmin()) {
            return true;
        }

        return $waterway->user_id == user_id() && $waterway->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given waterway.
     *
     * @param UserPolicy $user
     * @param Waterway $waterway
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Waterway $waterway)
    {
        return $waterway->user_id == user_id() && $waterway->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given waterway.
     *
     * @param UserPolicy $user
     * @param Waterway $waterway
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Waterway $waterway)
    {
        if ($user->canDo('waterway.waterway.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given waterway.
     *
     * @param UserPolicy $user
     * @param Waterway $waterway
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Waterway $waterway)
    {
        if ($user->canDo('waterway.waterway.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
